<?php

/**
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Chill\FormTypesBundle\Service;

use libphonenumber\PhoneNumber;
use libphonenumber\PhoneNumberFormat;
use libphonenumber\PhoneNumberUtil;
use Psr\Log\LoggerInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

use function in_array;

final class PhoneNumberHelper implements PhoneNumberHelperInterface
{
    private const FORMAT_URI = 'https://lookups.twilio.com/v1/PhoneNumbers/%s';

    private const LOOKUP_URI = 'https://lookups.twilio.com/v1/PhoneNumbers/%s';

    private HttpClientInterface $client;

    private LoggerInterface $logger;

    private PhoneNumberUtil $phoneNumberUtil;

    public function __construct(
        HttpClientInterface $client,
        LoggerInterface $logger,
        PhoneNumberUtil $phoneNumberUtil
    ) {
        $this->logger = $logger;
        $this->client = $client;
        $this->phoneNumberUtil = $phoneNumberUtil;
    }

    public function format(string $phonenumber): string
    {
        return $this->performTwilioFormat($phonenumber);
    }

    public function getType(string $phonenumber): string
    {
        return $this->performTwilioLookup($phonenumber) ?? 'unknown';
    }

    public function isTwilioEnabled(): bool
    {
        try {
            $response = $this
                ->client
                ->request(
                    'GET',
                    'https://api.twilio.com/2010-04-01/Accounts.json'
                );
        } catch (TransportExceptionInterface $e) {
            return false;
        }

        if (200 !== $response->getStatusCode()) {
            return false;
        }

        return true;
    }

    public function isValidPhonenumberAny(PhoneNumber $phonenumber): bool
    {
        $phone = $this->phoneNumberUtil->format($phonenumber, PhoneNumberFormat::INTERNATIONAL);

        $validation = $this->performTwilioLookup($phone);

        return in_array($validation, ['landline', 'voip', 'mobile'], true);
    }

    public function isValidPhonenumberLandOrVoip(PhoneNumber $phonenumber): bool
    {
        $phone = $this->phoneNumberUtil->format($phonenumber, PhoneNumberFormat::INTERNATIONAL);

        $validation = $this->performTwilioLookup($phone);

        return in_array($validation, ['landline', 'voip'], true);
    }

    public function isValidPhonenumberMobile(PhoneNumber $phonenumber): bool
    {
        $phone = $this->phoneNumberUtil->format($phonenumber, PhoneNumberFormat::INTERNATIONAL);

        $validation = $this->performTwilioLookup($phone);

        return 'mobile' === $validation;
    }

    private function performTwilioFormat(string $phonenumber): string
    {
        // filter only number
        $filtered = preg_replace('/[^0-9]/', '', $phonenumber);

        try {
            $response = $this
                ->client
                ->request(
                    'GET',
                    sprintf(self::FORMAT_URI, '+' . $filtered),
                );
        } catch (TransportExceptionInterface $e) {
            $this
                ->logger
                ->error(
                    'Could not format number due to server error',
                    [
                        'message' => $e->getMessage(),
                        'status_code' => $e->getCode(),
                        'phonenumber' => $phonenumber,
                    ]
                );

            return $phonenumber;
        }

        if (200 !== $response->getStatusCode()) {
            $this
                ->logger
                ->error(
                    'Could not format number due to client error',
                    [
                        'message' => $response->getContent(false),
                        'status_code' => $response->getStatusCode(),
                        'phonenumber' => $phonenumber,
                    ]
                );

            return $phonenumber;
        }

        return $response->toArray()['national_format'];
    }

    private function performTwilioLookup(string $phonenumber): ?string
    {
        $filtered = preg_replace('/[^0-9]/', '', $phonenumber);

        try {
            $response = $this
                ->client
                ->request(
                    'GET',
                    sprintf(self::LOOKUP_URI, '+' . $filtered),
                    [
                        'query' => [
                            'Type' => 'carrier',
                        ],
                    ]
                );
        } catch (TransportExceptionInterface $e) {
            $this
                ->logger
                ->error(
                    'Could not lookup number due to server error',
                    [
                        'message' => $e->getMessage(),
                        'status_code' => $e->getCode(),
                        'phonenumber' => $phonenumber,
                    ]
                );

            return $phonenumber;
        }

        if (200 !== $response->getStatusCode()) {
            $this
                ->logger
                ->error(
                    'Could not lookup number due to client error',
                    [
                        'message' => $response->getContent(false),
                        'status_code' => $response->getStatusCode(),
                        'phonenumber' => $phonenumber,
                    ]
                );

            return $phonenumber;
        }

        return $response->toArray()['carrier']['type'];
    }
}
