<?php

/**
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

declare(strict_types=1);

namespace Chill\FormTypesBundle\Tests\Form\Types;

use Chill\FormTypesBundle\Form\Types\PhoneNumberType;
use libphonenumber\PhoneNumberFormat;
use Locale;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Form\FormFactoryInterface;
use Symfony\Component\Form\Forms;

/**
 * @internal
 *
 * @covers \Chill\FormTypesBundle\Form\DataTransformer\PhoneNumberTypeTransformer
 * @covers \Chill\FormTypesBundle\Form\Types\PhoneNumberType
 */
final class PhoneNumberTypeTest extends TestCase
{
    /**
     * @var FormFactoryInterface
     */
    protected $factory;

    protected function setUp(): void
    {
        Locale::setDefault('en');

        $this->factory = Forms::createFormFactoryBuilder()->getFormFactory();
    }

    public function singleFieldProvider()
    {
        yield ['0,-03f212@%3g4 g5k 6789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 123456789'];

        yield ['0032 123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 123456789'];

        yield ['0032 123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::NATIONAL], '123456789'];

        yield ['+32 123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 123456789'];

        yield ['+32 123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::NATIONAL], '123456789'];

        yield ['0123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 123456789'];

        yield ['0123456789', ['default_region' => 'BE', 'format' => PhoneNumberFormat::NATIONAL], '123456789'];

        yield ['0486/540660', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 486 54 06 60'];

        yield ['0486/540660', ['default_region' => 'BE', 'format' => PhoneNumberFormat::NATIONAL], '0486 54 06 60'];

        yield ['0486/540.660', ['default_region' => 'BE', 'format' => PhoneNumberFormat::INTERNATIONAL], '+32 486 54 06 60'];

        yield ['0486/540.660', ['default_region' => 'BE', 'format' => PhoneNumberFormat::NATIONAL], '0486 54 06 60'];

        yield ['+441234567890', [], '+44 1234 567890'];

        yield ['+44 1234 567890', ['format' => PhoneNumberFormat::NATIONAL], '+44 1234 567890'];

        yield ['+44 1234 567890', ['default_region' => 'GB', 'format' => PhoneNumberFormat::NATIONAL], '01234 567890'];

        yield ['+1 650-253-0000', ['default_region' => 'GB', 'format' => PhoneNumberFormat::NATIONAL], '00 1 650-253-0000'];

        yield ['01234 567890', ['default_region' => 'GB'], '+44 1234 567890'];

        yield ['', [], ''];
    }

    public function testGetNameAndBlockPrefixAreTel()
    {
        $type = new PhoneNumberType();

        self::assertSame('chill_formtypes_bundle', $type->getBlockPrefix());
    }

    /**
     * @dataProvider singleFieldProvider
     */
    public function testSingleField(string $input, array $options, string $output)
    {
        $form = $this->factory->create(PhoneNumberType::class, null, $options);

        $form->submit($input);

        if (method_exists($form, 'getTransformationFailure') && $failure = $form->getTransformationFailure()) {
            throw $failure;
        }
        self::assertTrue($form->isSynchronized());

        $view = $form->createView();

        self::assertSame('tel', $view->vars['type']);
        self::assertSame($output, $view->vars['value']);
    }
}
